==> 0.0.4.tar.gz: lib/package/cascader_checkbox.dart <==
import 'package:flutter/material.dart';
import './checkbox.dart';

class MillCascaderCheckbox extends StatefulWidget {
  MillCascaderCheckbox({
    Key key,
    @required this.tree,
    this.defaultValue = const [],
    this.alasLabel = 'label',
    this.alasValue = 'value',
    this.width,
    this.stack = 3,
    this.size = 22,
    this.activeColor = const Color(0xfff6f6f6),
    this.onChange,
  }) : super(key: key);
  final List tree;
  final List defaultValue;
  final String alasLabel;
  final String alasValue;
  final double width;
  final int stack;
  final double size;
  final Color activeColor;
  final Function onChange;
  _MillCascaderCheckbox createState() => new _MillCascaderCheckbox();
}

class _MillCascaderCheckbox extends State<MillCascaderCheckbox> {
  List _tree;
  List colums = [];
  double _width = 390;
  @override
  void initState() {
    if (widget.width != null) {
      _width = widget.width;
    }

    _tree = _initData(widget.tree);
    _select(0, _tree);
    super.initState();
  }

  _initData(list, {int state, int index = 0, bool disabled}) {
    List nlist = [];
    for (var i = 0; i < list.length; i++) {
      var checked = widget.defaultValue.indexOf(list[i][widget.alasValue]) > -1
          ? 1
          : state != null
              ? state
              : 0;
      bool isDisabled = list[i]['disabled'] != null
          ? list[i]['disabled']
          : disabled != null
              ? disabled
              : false;
      Map<String, dynamic> result = {
        'checked': checked,
        '${widget.alasLabel}': list[i][widget.alasLabel],
        '${widget.alasValue}': list[i][widget.alasValue],
        'stack': index,
        'disabled': isDisabled,
      };
      int newIndex = index + 1;
      if (list[i]['children'] != null &&
          list[i]['children'].length > 0 &&
          newIndex <= widget.stack) {
        result['children'] = _initData(list[i]['children'],
            disabled: isDisabled, state: checked, index: newIndex);
      } else {
        result['children'] = [];
      }

      var selected = result['children'].where((val) => val['checked'] == 1);
      var im = result['children'].where((val) => val['checked'] == 2);
      if (selected.length > 0) {
        result['checked'] =
            selected.length == result['children'].length ? 1 : 2;
      } else if (im.length > 0) {
        result['checked'] = 2;
      }

      nlist.add(result);
    }
    return nlist;
  }

  _change(data) {
    colums[data['stack']] = data[widget.alasValue];
    var tree = _getData(colums, data['stack'] + 1);
    _select(data['stack'] + 1, tree);
    setState(() {
      colums = colums;
    });
  }

  _select(index, data) {
    while (index < widget.stack) {
      if (data != null) {
        var firstIndex;
        for (var i = 0; i < data.length; i++) {
          if (firstIndex == null &&
              data[i]['checked'] != 0 &&
              data[i]['children'] != null) {
            firstIndex = i;
          }
        }

        if (firstIndex == null && data.length > 0) {
          firstIndex = 0;
        }

        if (firstIndex != null) {
          if (colums.length <= index) {
            colums.add(data[firstIndex][widget.alasValue]);
          } else {
            colums[index] = data[firstIndex][widget.alasValue];
          }
          data = _getData(colums, index + 1);
        } else if (colums.length > index) {
          colums[index] = null;
        }
        index++;
      } else {
        index = widget.stack;
      }
    }
  }

  _getData(keys, last) {
    var data = _tree;
    var i = 0;
    while (i < last) {
      int index =
          data.indexWhere((element) => element[widget.alasValue] == keys[i]);
      data = data[index]['children'];
      i++;
    }
    return data;
  }

  _changeState(data) {
    if (!data['disabled']) {
      int state = data['checked'] != 1 ? 1 : 0;
      var tree = _setTreeState(data, state);
      while (tree['stack'] > 0) {
        tree = _setFtree(tree);
      }
      int index = _tree.indexWhere(
          (element) => element[widget.alasValue] == tree[widget.alasValue]);
      _tree[index] = tree;
      setState(() {
        _tree = _tree;
      });
      if (widget.onChange != null) {
        widget.onChange(_tree);
      }
    }
  }

  _setFtree(data) {
    var ftree = _tree;
    var i = 0;
    for (; i < data['stack'] - 1; i++) {
      var fd = ftree.firstWhere((item) => item[widget.alasValue] == colums[i]);
      ftree = fd['children'];
    }
    var fdata = ftree.firstWhere((item) => item[widget.alasValue] == colums[i]);
    var index = fdata['children']
        .indexWhere((el) => el[widget.alasValue] == data[widget.alasValue]);
    fdata['children'][index] = data;
    var selected = fdata['children'].where((val) => val['checked'] == 1);
    var im = fdata['children'].where((val) => val['checked'] == 2);
    if (selected.length > 0) {
      fdata['checked'] = selected.length == fdata['children'].length ? 1 : 2;
    } else if (im.length > 0) {
      fdata['checked'] = 2;
    }
    return fdata;
  }

  _setTreeState(data, state) {
    Map<String, dynamic> result = {
      'checked': state,
      'stack': data['stack'],
      '${widget.alasLabel}': data[widget.alasLabel],
      '${widget.alasValue}': data[widget.alasValue],
      'disabled': data['disabled'],
    };
    if (data['children'] != null && data['children'].length > 0) {
      result['children'] =
          data['children'].map((el) => _setTreeState(el, state)).toList();
    } else {
      result['children'] = [];
    }
    return result;
  }

  _getListByKey(key) {
    int index = colums.indexOf(key);
    int i = 0;
    List data = _tree;
    while (i < index) {
      var maps =
          data.firstWhere((element) => element[widget.alasValue] == colums[i]);
      data = maps['children'];
      i++;
    }
    return data;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      width: _width,
      color: Color(0xffffffff),
      child: Row(
        children: colums
            .map((item) => Container(
                  width: _width / widget.stack,
                  decoration: BoxDecoration(
                      border: Border(
                          right: BorderSide(
                              width: 1.0,
                              color: Color(
                                  colums.indexOf(item) < colums.length - 1
                                      ? 0xfff3f3f3
                                      : 0xffffffff)))),
                  child: ListView(
                    children: _getListByKey(item).map<Widget>((el) {
                      return GestureDetector(
                        onTap: () {
                          _change(el);
                        },
                        child: Container(
                            width: _width / widget.stack,
                            padding: EdgeInsets.only(
                                top: 12, bottom: 12, left: 8, right: 8),
                            color:
                                colums[el['stack']] == el[widget.alasValue] &&
                                        colums.indexOf(item) < colums.length - 1
                                    ? widget.activeColor
                                    : Color(0xffffffff),
                            child: Row(
                              children: <Widget>[
                                MillCheckBox(
                                  disabled: el['disabled'],
                                  size: widget.size,
                                  outside: 8,
                                  value: el['checked'] == 1,
                                  indeterminate: el['checked'] == 2,
                                  onChange: () {
                                    _changeState(el);
                                  },
                                ),
                                Padding(
                                  padding: EdgeInsets.only(left: 10),
                                ),
                                Expanded(
                                  flex: 1,
                                  child: Text(
                                    el[widget.alasLabel],
                                    overflow: TextOverflow.ellipsis,
                                    style:
                                        TextStyle(fontSize: widget.size * 0.7),
                                  ),
                                ),
                              ],
                            )),
                      );
                    }).toList(),
                  ),
                ))
            .toList(),
      ),
    );
  }
}

==> 0.0.7.tar.gz: lib/package/cascader_checkbox.dart <==
import 'package:flutter/material.dart';
import './checkbox.dart';

class MillCascaderCheckbox extends StatefulWidget {
  MillCascaderCheckbox({
    Key key,
    @required this.tree,
    this.defaultValue = const [],
    this.aliasLabel = 'label',
    this.aliasValue = 'value',
    this.width,
    this.stack = 3,
    this.size = 22,
    this.activeColor = const Color(0xfff6f6f6),
    this.itemPadding =
        const EdgeInsets.only(top: 12, bottom: 12, left: 8, right: 8),
    this.onChange,
  }) : super(key: key);
  final List tree;
  final List defaultValue;
  final String aliasLabel;
  final String aliasValue;
  final double width;
  final int stack;
  final double size;
  final Color activeColor;
  final Function onChange;
  final EdgeInsets itemPadding;
  _MillCascaderCheckbox createState() => new _MillCascaderCheckbox();
}

class _MillCascaderCheckbox extends State<MillCascaderCheckbox> {
  List _tree;
  List colums = [];
  double _width = 390;
  @override
  void initState() {
    if (widget.width != null) {
      _width = widget.width;
    }

    _tree = _initData(widget.tree);
    _select(0, _tree);
    super.initState();
  }

  _initData(list, {int state, int index = 0, bool disabled}) {
    List nlist = [];
    for (var i = 0; i < list.length; i++) {
      var checked = widget.defaultValue.indexOf(list[i][widget.aliasValue]) > -1
          ? 1
          : state != null
              ? state
              : 0;
      bool isDisabled = list[i]['disabled'] != null
          ? list[i]['disabled']
          : disabled != null
              ? disabled
              : false;
      Map<String, dynamic> result = {
        'checked': checked,
        '${widget.aliasLabel}': list[i][widget.aliasLabel],
        '${widget.aliasValue}': list[i][widget.aliasValue],
        'stack': index,
        'disabled': isDisabled,
      };
      int newIndex = index + 1;
      if (list[i]['children'] != null &&
          list[i]['children'].length > 0 &&
          newIndex <= widget.stack) {
        result['children'] = _initData(list[i]['children'],
            disabled: isDisabled, state: checked, index: newIndex);
      } else {
        result['children'] = [];
      }

      var selected = result['children'].where((val) => val['checked'] == 1);
      var im = result['children'].where((val) => val['checked'] == 2);
      if (selected.length > 0) {
        result['checked'] =
            selected.length == result['children'].length ? 1 : 2;
      } else if (im.length > 0) {
        result['checked'] = 2;
      }

      nlist.add(result);
    }
    return nlist;
  }

  _change(data) {
    colums[data['stack']] = data[widget.aliasValue];
    var tree = _getData(colums, data['stack'] + 1);
    _select(data['stack'] + 1, tree);
    setState(() {
      colums = colums;
    });
  }

  _select(index, data) {
    while (index < widget.stack) {
      if (data != null) {
        var firstIndex;
        for (var i = 0; i < data.length; i++) {
          if (firstIndex == null &&
              data[i]['checked'] != 0 &&
              data[i]['children'] != null) {
            firstIndex = i;
          }
        }

        if (firstIndex == null && data.length > 0) {
          firstIndex = 0;
        }

        if (firstIndex != null) {
          if (colums.length <= index) {
            colums.add(data[firstIndex][widget.aliasValue]);
          } else {
            colums[index] = data[firstIndex][widget.aliasValue];
          }
          data = _getData(colums, index + 1);
        } else if (colums.length > index) {
          colums[index] = null;
        }
        index++;
      } else {
        index = widget.stack;
      }
    }
  }

  _getData(keys, last) {
    var data = _tree;
    var i = 0;
    while (i < last) {
      int index =
          data.indexWhere((element) => element[widget.aliasValue] == keys[i]);
      data = data[index]['children'];
      i++;
    }
    return data;
  }

  _changeState(data) {
    if (!data['disabled']) {
      int state = data['checked'] != 1 ? 1 : 0;
      var tree = _setTreeState(data, state);
      while (tree['stack'] > 0) {
        tree = _setFtree(tree);
      }
      int index = _tree.indexWhere(
          (element) => element[widget.aliasValue] == tree[widget.aliasValue]);
      _tree[index] = tree;
      setState(() {
        _tree = _tree;
      });
      if (widget.onChange != null) {
        widget.onChange(_tree);
      }
    }
  }

  _setFtree(data) {
    var ftree = _tree;
    var i = 0;
    for (; i < data['stack'] - 1; i++) {
      var fd = ftree.firstWhere((item) => item[widget.aliasValue] == colums[i]);
      ftree = fd['children'];
    }
    var fdata =
        ftree.firstWhere((item) => item[widget.aliasValue] == colums[i]);
    var index = fdata['children']
        .indexWhere((el) => el[widget.aliasValue] == data[widget.aliasValue]);
    fdata['children'][index] = data;
    var selected = fdata['children'].where((val) => val['checked'] == 1);
    var im = fdata['children'].where((val) => val['checked'] == 2);
    if (selected.length > 0) {
      fdata['checked'] = selected.length == fdata['children'].length ? 1 : 2;
    } else if (im.length > 0) {
      fdata['checked'] = 2;
    } else {
      fdata['checked'] = 0;
    }
    return fdata;
  }

  _setTreeState(data, state) {
    Map<String, dynamic> result = {
      'checked': state,
      'stack': data['stack'],
      '${widget.aliasLabel}': data[widget.aliasLabel],
      '${widget.aliasValue}': data[widget.aliasValue],
      'disabled': data['disabled'],
    };
    if (data['children'] != null && data['children'].length > 0) {
      result['children'] =
          data['children'].map((el) => _setTreeState(el, state)).toList();
    } else {
      result['children'] = [];
    }
    return result;
  }

  _getListByKey(key) {
    int index = colums.indexOf(key);
    int i = 0;
    List data = _tree;
    while (i < index) {
      var maps =
          data.firstWhere((element) => element[widget.aliasValue] == colums[i]);
      data = maps['children'];
      i++;
    }
    return data;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      width: _width,
      color: Color(0xffffffff),
      child: Row(
        children: colums
            .map((item) => Container(
                  width: _width / widget.stack,
                  decoration: BoxDecoration(
                    border: Border(
                      right: BorderSide(
                        width: 1.0,
                        color: Color(colums.indexOf(item) < colums.length - 1
                            ? 0xfff3f3f3
                            : 0xffffffff),
                      ),
                    ),
                  ),
                  child: ListView(
                    padding: EdgeInsets.all(0),
                    children: _getListByKey(item).map<Widget>((el) {
                      return GestureDetector(
                        behavior: HitTestBehavior.opaque,
                        onTap: () {
                          _change(el);
                        },
                        child: Container(
                            width: _width / widget.stack,
                            padding: widget.itemPadding,
                            color:
                                colums[el['stack']] == el[widget.aliasValue] &&
                                        colums.indexOf(item) < colums.length - 1
                                    ? widget.activeColor
                                    : Color(0xffffffff),
                            child: Row(
                              children: <Widget>[
                                MillCheckBox(
                                  disabled: el['disabled'],
                                  size: widget.size,
                                  outside: 8,
                                  value: el['checked'] == 1,
                                  indeterminate: el['checked'] == 2,
                                  onChange: () {
                                    _changeState(el);
                                  },
                                ),
                                Padding(
                                  padding: EdgeInsets.only(left: 10),
                                ),
                                Expanded(
                                  flex: 1,
                                  child: Text(
                                    el[widget.aliasLabel],
                                    overflow: TextOverflow.ellipsis,
                                    style:
                                        TextStyle(fontSize: widget.size * 0.7),
                                  ),
                                ),
                              ],
                            )),
                      );
                    }).toList(),
                  ),
                ))
            .toList(),
      ),
    );
  }
}
